@@ -2,14 +2,12 @@
 
 require_relative "bootsnap/version"
 require_relative "bootsnap/bundler"
-require_relative "bootsnap/load_path_cache"
-require_relative "bootsnap/compile_cache"
 
 module Bootsnap
   InvalidConfiguration = Class.new(StandardError)
 
   class << self
-    attr_reader :logger
+    attr_reader :cache_dir, :logger
 
     def log_stats!
       stats = {hit: 0, revalidated: 0, miss: 0, stale: 0}
@@ -54,11 +52,17 @@ module Bootsnap
       revalidation: false,
       compile_cache_iseq: true,
       compile_cache_yaml: true,
-      compile_cache_json: true
+      compile_cache_json: (compile_cache_json_unset = true)
     )
+      unless compile_cache_json_unset
+        warn("Bootsnap.setup `compile_cache_json` argument is deprecated and has no effect")
+      end
+
+      @cache_dir = "#{cache_dir}/bootsnap"
+
       if load_path_cache
         Bootsnap::LoadPathCache.setup(
-          cache_path: "#{cache_dir}/bootsnap/load-path-cache",
+          cache_path: "#{@cache_dir}/load-path-cache",
           development_mode: development_mode,
           ignore_directories: ignore_directories,
           readonly: readonly,
@@ -66,10 +70,9 @@ module Bootsnap
       end
 
       Bootsnap::CompileCache.setup(
-        cache_dir: "#{cache_dir}/bootsnap/compile-cache",
+        cache_dir: "#{@cache_dir}/compile-cache",
         iseq: compile_cache_iseq,
         yaml: compile_cache_yaml,
-        json: compile_cache_json,
         readonly: readonly,
         revalidation: revalidation,
       )
@@ -83,7 +86,7 @@ module Bootsnap
       env = ENV["RAILS_ENV"] || ENV["RACK_ENV"] || ENV["ENV"]
       development_mode = ["", nil, "development"].include?(env)
 
-      unless ENV["DISABLE_BOOTSNAP"]
+      if enabled?("BOOTSNAP")
         cache_dir = ENV["BOOTSNAP_CACHE_DIR"]
         unless cache_dir
           config_dir_frame = caller.detect do |line|
@@ -112,11 +115,11 @@ module Bootsnap
         setup(
           cache_dir: cache_dir,
           development_mode: development_mode,
-          load_path_cache: !ENV["DISABLE_BOOTSNAP_LOAD_PATH_CACHE"],
-          compile_cache_iseq: !ENV["DISABLE_BOOTSNAP_COMPILE_CACHE"],
-          compile_cache_yaml: !ENV["DISABLE_BOOTSNAP_COMPILE_CACHE"],
-          compile_cache_json: !ENV["DISABLE_BOOTSNAP_COMPILE_CACHE"],
-          readonly: !!ENV["BOOTSNAP_READONLY"],
+          load_path_cache: enabled?("BOOTSNAP_LOAD_PATH_CACHE"),
+          compile_cache_iseq: enabled?("BOOTSNAP_COMPILE_CACHE"),
+          compile_cache_yaml: enabled?("BOOTSNAP_COMPILE_CACHE"),
+          readonly: bool_env("BOOTSNAP_READONLY"),
+          revalidation: bool_env("BOOTSNAP_REVALIDATE"),
           ignore_directories: ignore_directories,
         )
 
@@ -148,5 +151,19 @@ module Bootsnap
 
     # Allow the C extension to redefine `rb_get_path` without warning.
     alias_method :rb_get_path, :rb_get_path # rubocop:disable Lint/DuplicateMethods
+
+    private
+
+    def enabled?(key)
+      !ENV["DISABLE_#{key}"]
+    end
+
+    def bool_env(key, default: false)
+      value = ENV.fetch(key) { default }
+      !["0", "false", false].include?(value)
+    end
   end
 end
+
+require_relative "bootsnap/compile_cache"
+require_relative "bootsnap/load_path_cache"
