@@ -1,11 +1,11 @@
 # frozen_string_literal: true
 
 module Kernel
-  module_function
+  alias_method :require_without_bootsnap, :require
 
-  alias_method(:require_without_bootsnap, :require)
+  alias_method :require, :require # Avoid method redefinition warnings
 
-  def require(path)
+  def require(path) # rubocop:disable Lint/DuplicateMethods
     return require_without_bootsnap(path) unless Bootsnap::LoadPathCache.enabled?
 
     string_path = Bootsnap.rb_get_path(path)
@@ -15,8 +15,11 @@ module Kernel
     if Bootsnap::LoadPathCache::FALLBACK_SCAN.equal?(resolved)
       if (cursor = Bootsnap::LoadPathCache.loaded_features_index.cursor(string_path))
         ret = require_without_bootsnap(path)
-        resolved = Bootsnap::LoadPathCache.loaded_features_index.identify(string_path, cursor)
-        Bootsnap::LoadPathCache.loaded_features_index.register(string_path, resolved)
+
+        # The file we required may have unloaded the cache
+        resolved = Bootsnap::LoadPathCache.loaded_features_index&.identify(string_path, cursor)
+        Bootsnap::LoadPathCache.loaded_features_index&.register(string_path, resolved)
+
         return ret
       else
         return require_without_bootsnap(path)
@@ -24,14 +27,16 @@ module Kernel
     elsif false == resolved
       return false
     elsif resolved.nil?
-      error = LoadError.new(+"cannot load such file -- #{path}")
-      error.instance_variable_set(:@path, path)
-      raise error
+      return require_without_bootsnap(path)
     else
       # Note that require registers to $LOADED_FEATURES while load does not.
       ret = require_without_bootsnap(resolved)
-      Bootsnap::LoadPathCache.loaded_features_index.register(string_path, resolved)
+
+      # The file we required may have unloaded the cache
+      Bootsnap::LoadPathCache.loaded_features_index&.register(string_path, resolved)
       return ret
     end
   end
+
+  private :require
 end
