@@ -21,7 +21,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -39,7 +38,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -57,7 +55,6 @@ module Bootsnap
         load_path_cache: false,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -75,7 +72,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: false,
         compile_cache_yaml: false,
-        compile_cache_json: false,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -100,7 +96,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -119,7 +114,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: %w[foo bar],
         readonly: false,
         revalidation: false,
@@ -137,7 +131,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: true,
         revalidation: false,
@@ -153,7 +146,6 @@ module Bootsnap
         load_path_cache: true,
         compile_cache_iseq: true,
         compile_cache_yaml: true,
-        compile_cache_json: true,
         ignore_directories: nil,
         readonly: false,
         revalidation: false,
@@ -162,6 +154,20 @@ module Bootsnap
       Bootsnap.default_setup
     end
 
+    def test_setup_appends_bootsnap_to_cache_dir
+      skip("Need a working Process.fork to test in isolation") unless Process.respond_to?(:fork)
+
+      pid = fork do
+        Bootsnap.setup(cache_dir: @tmp_dir)
+
+        assert_equal("#{@tmp_dir}/bootsnap", Bootsnap.cache_dir)
+      end
+      _, status = Process.waitpid2(pid)
+      assert_predicate status, :success?
+    ensure
+      Bootsnap.unload_cache!
+    end
+
     def test_unload_cache
       Bootsnap.unload_cache!
     end
